@@ -1,6 +1,6 @@
 /* Copyright (C) 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: ctrlvalue.h,v 1.2 2004/06/16 21:11:52 mmj Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -25,5 +25,7 @@
 #define CTRLVALUE_H
 
 char *ctrlvalue(const char *listdir, const char *ctrlstr);
+char *ctrlcontent(const char *listdir, const char *ctrlstr);
+char *textcontent(const char *listdir, const char *ctrlstr);
 
 #endif /* CTRLVALUE_H */
