@@ -1,6 +1,6 @@
 /* Copyright (C) 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: send_help.h,v 1.4 2006/12/06 21:29:27 mortenp Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -24,7 +24,7 @@
 #ifndef SEND_HELP_H
 #define SEND_HELP_H
 
-void send_help(const char *listdir, const char *emailaddr,
-	       const char *mlmmjsend, const char *name, const char *textfile);
+void send_help(const char *listdir, const char *queuefilename,
+		const char *emailaddr, const char *mlmmjsend);
 
 #endif
