@@ -1,6 +1,6 @@
 /* Copyright (C) 2004 Morten K. Poulsen <morten at afdelingp.dk>
  *
- * $Id: mlmmj-bounce.c,v 1.44 2008/10/31 14:20:51 mortenp Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -49,68 +49,13 @@
 #include "find_email_adr.h"
 #include "gethdrline.h"
 
-char *fetchindexes(const char *bouncefile)
-{
-	int fd;
-	char *indexstr = NULL, *s, *start, *line, *cur, *colon, *next;
-	size_t len;
-	struct stat st;
-
-	fd = open(bouncefile, O_RDONLY);
-	if(fd < 0) {
-		log_error(LOG_ARGS, "Could not open bounceindexfile %s",
-				bouncefile);
-		return NULL;
-	}
-
-	if(fstat(fd, &st) < 0) {
-		log_error(LOG_ARGS, "Could not open bounceindexfile %s",
-					bouncefile);
-	}
-
-	if(st.st_size == 0) {
-		log_error(LOG_ARGS, "Bounceindexfile %s is size 0",
-					bouncefile);
-		return NULL;
-	}
-
-	start = mmap(0, st.st_size, PROT_READ, MAP_SHARED, fd, 0);
-	if(start == MAP_FAILED) {
-		log_error(LOG_ARGS, "Could not mmap bounceindexfile");
-		return NULL;
-	}
-
-
-	for(next = cur = start; next < start + st.st_size; next++) {
-		if(*next == '\n') {
-			len = next - cur;
-			line = mymalloc(len + 1);
-			strncpy(line, cur, len);
-			line[len] = '\0';
-			cur = next + 1;
-		} else
-	 		continue;
-
-		colon = strchr(line, ':');
-		MY_ASSERT(colon);
-		*colon = '\0';
-		s = indexstr;
-		indexstr = concatstr(4, s, "        ", line, "\n");
-		myfree(s);
-		myfree(line);
-	}
-
-	munmap(start, st.st_size);
-	close(fd);
-
-	return indexstr;
-}
 
 void do_probe(const char *listdir, const char *mlmmjsend, const char *addr)
 {
-	char *myaddr, *from, *a, *indexstr, *queuefilename, *listaddr;
+	text *txt;
+	file_lines_state *fls;
+	char *myaddr, *from, *a, *queuefilename, *listaddr;
 	char *listfqdn, *listname, *probefile, *listdelim=getlistdelim(listdir);
-	char *maildata[] = { "bouncenumbers", NULL };
 	int fd;
 	time_t t;
 
@@ -138,17 +83,17 @@ void do_probe(const char *listdir, const char *mlmmjsend, const char *addr)
 	}
 	*a = '@';
 
-	indexstr = fetchindexes(addr);
-	if(indexstr == NULL) {
-		log_error(LOG_ARGS, "Could not fetch bounceindexes");
-		exit(EXIT_FAILURE);
-	}
-
-	maildata[1] = indexstr;
-	queuefilename = prepstdreply(listdir, "bounce-probe", "$listowner$",
-					myaddr, NULL, 1, maildata, NULL, NULL);
+	txt = open_text(listdir, "probe", NULL, NULL, NULL, "bounce-probe");
+	MY_ASSERT(txt);
+	register_unformatted(txt, "bouncenumbers", "%bouncenumbers%"); /* DEPRECATED */
+	fls = init_truncated_file_lines(addr, 0, ':');
+	register_formatted(txt, "bouncenumbers",
+			rewind_file_lines, get_file_line, fls);
+	queuefilename = prepstdreply(txt, listdir, "$listowner$", myaddr, NULL);
 	MY_ASSERT(queuefilename);
-	myfree(indexstr);
+	close_text(txt);
+
+	finish_file_lines(fls);
 
 	probefile = concatstr(4, listdir, "/bounce/", addr, "-probe");
 	MY_ASSERT(probefile);
@@ -223,13 +168,14 @@ char *dsnparseaddr(const char *mailname)
 
 static void print_help(const char *prg)
 {
-	printf("Usage: %s -L /path/to/list -a john=doe.org [-n num | -p]\n"
-		" -a: Address string that bounces\n"
-		" -h: This help\n"
-		" -L: Full path to list directory\n"
-		" -n: Message number in the archive\n"
-		" -p: Send out a probe\n"
-		" -V: Print version\n", prg);
+	printf("Usage: %s -L /path/to/list\n"
+	       "       -a john=doe.org [-n num | -p]\n"
+	       " -a: Address string that bounces\n"
+	       " -h: This help\n"
+	       " -L: Full path to list directory\n"
+	       " -n: Message number in the archive\n"
+	       " -p: Send out a probe\n"
+	       " -V: Print version\n", prg);
 
 	exit(EXIT_SUCCESS);
 }
@@ -396,7 +342,7 @@ int main(int argc, char **argv)
 	*a = '@';
 
 	/* make sure it's a subscribed address */
-	if(is_subbed(listdir, address)) {
+	if(is_subbed(listdir, address, 0) == SUB_NONE) {
 		log_error(LOG_ARGS, "%s is bouncing but not subscribed?",
 				address);
 		if(mailname)
