@@ -1,6 +1,6 @@
 /* Copyright (C) 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-maintd.c,v 1.62 2009/12/21 06:32:00 mortenp Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -315,6 +315,9 @@ int resend_queue(const char *listdir, const char *mlmmjsend)
 		if(t - st.st_mtime > bouncelife) {
 			unlink(mailname);
 			myfree(mailname);
+			myfree(from);
+			myfree(to);
+			myfree(repto);
 			continue;
 		}
 
@@ -356,6 +359,10 @@ int resend_queue(const char *listdir, const char *mlmmjsend)
 						"-T", to,
 						"-a", (char *)NULL);
 			}
+			log_error(LOG_ARGS, "Could not execlp %s",
+						mlmmjsend);
+			/* This is the child. Exit on failure. */
+			exit(EXIT_FAILURE);
 		}
 	}
 
@@ -474,6 +481,10 @@ int resend_requeue(const char *listdir, const char *mlmmjsend)
 					"-s", subnewname,
 					"-a",
 					"-D", (char *)NULL);
+			log_error(LOG_ARGS, "Could not execlp %s",
+						mlmmjsend);
+			/* This is the child. Exit on failure. */
+			exit(EXIT_FAILURE);
 		}
 	}
 
@@ -513,20 +524,27 @@ int clean_nolongerbouncing(const char *listdir)
 		   (strcmp(dp->d_name, ".") == 0))
 				continue;
 
-		if(stat(dp->d_name, &st) < 0) {
-			log_error(LOG_ARGS, "Could not stat(%s)", dp->d_name);
-			continue;
-		}
-
 		filename = mystrdup(dp->d_name);
 
-		if((s = strstr(filename, "-probe"))) {
+		s = strrchr(filename, '-');
+		if(s && (strcmp(s, "-probe") == 0)) {
+			if(stat(filename, &st) < 0) {
+				log_error(LOG_ARGS, "Could not stat(%s)",
+					  filename);
+				myfree(filename);
+				continue;
+			}
+
 			probefd = open(filename, O_RDONLY);
-			if(probefd < 0)
+			if(probefd < 0) {
+				myfree(filename);
 				continue;
+			}
 			probetimestr = mygetline(probefd);
-			if(probetimestr == NULL)
+			if(probetimestr == NULL) {
+				myfree(filename);
 				continue;
+			}
 			close(probefd);
 			chomp(probetimestr);
 			probetime = (time_t)strtol(probetimestr, NULL, 10);
@@ -534,6 +552,7 @@ int clean_nolongerbouncing(const char *listdir)
 			t = time(NULL);
 			if(t - probetime > WAITPROBE) {
 				unlink(filename);
+				/* remove -probe onwards from filename */
 				*s = '\0';
 				unlink(filename);
 				s = concatstr(2, filename, ".lastmsg");
@@ -578,18 +597,19 @@ int probe_bouncers(const char *listdir, const char *mlmmjbounce)
 		   (strcmp(dp->d_name, ".") == 0))
 				continue;
 
-		if(stat(dp->d_name, &st) < 0) {
-			log_error(LOG_ARGS, "Could not stat(%s)", dp->d_name);
-			continue;
-		}
-		
-		if(strstr(dp->d_name, "-probe"))
+		s = strrchr(dp->d_name, '-');
+		if(s && (strcmp(s, "-probe") == 0))
 			continue;
 
 		s = strrchr(dp->d_name, '.');
 		if(s && (strcmp(s, ".lastmsg") == 0))
 			continue;
 			
+		if(stat(dp->d_name, &st) < 0) {
+			log_error(LOG_ARGS, "Could not stat(%s)", dp->d_name);
+			continue;
+		}
+		
 		probefile = concatstr(2, dp->d_name, "-probe");
 		
 		/* Skip files which already have a probe out */
@@ -618,7 +638,8 @@ int probe_bouncers(const char *listdir, const char *mlmmjbounce)
 					"-p", (char *)NULL);
 			log_error(LOG_ARGS, "Could not execlp %s",
 						mlmmjbounce);
-			return 1;
+			/* This is the child. Exit on failure. */
+			exit(EXIT_FAILURE);
 		}
 	}
 	closedir(bouncedir);
@@ -666,18 +687,19 @@ int unsub_bouncers(const char *listdir, const char *mlmmjunsub)
 		   (strcmp(dp->d_name, ".") == 0))
 				continue;
 
-		if(stat(dp->d_name, &st) < 0) {
-			log_error(LOG_ARGS, "Could not stat(%s)", dp->d_name);
-			continue;
-		}
-		
-		if(strstr(dp->d_name, "-probe"))
+		a = strrchr(dp->d_name, '-');
+		if(a && (strcmp(a, "-probe") == 0))
 			continue;
 
 		a = strrchr(dp->d_name, '.');
 		if(a && (strcmp(a, ".lastmsg") == 0))
 			continue;
 		
+		if(stat(dp->d_name, &st) < 0) {
+			log_error(LOG_ARGS, "Could not stat(%s)", dp->d_name);
+			continue;
+		}
+		
 		probefile = concatstr(2, dp->d_name, "-probe");
 		
 		/* Skip files which already have a probe out */
@@ -712,6 +734,7 @@ int unsub_bouncers(const char *listdir, const char *mlmmjunsub)
 		a = strchr(firstbounce, ':');
 		if(a == NULL) {
 			myfree(firstbounce);
+			myfree(bouncedata);
 			continue;
 		}
 
@@ -719,14 +742,17 @@ int unsub_bouncers(const char *listdir, const char *mlmmjunsub)
 		bouncetime = (time_t)strtol(a, NULL, 10);
 		myfree(firstbounce);
 		t = time(NULL);
-		if(t - bouncetime < bouncelife + WAITPROBE)
+		if(t - bouncetime < bouncelife + WAITPROBE) {
+			myfree(bouncedata);
 			continue; /* ok, don't unsub this one */
+		}
 		
 		/* Ok, go ahead and unsubscribe the address */
 		address = mystrdup(dp->d_name);
 		a = strchr(address, '=');
 		if(a == NULL) { /* skip malformed */
 			myfree(address);
+			myfree(bouncedata);
 			continue;
 		}
 		*a = '@';
@@ -735,6 +761,8 @@ int unsub_bouncers(const char *listdir, const char *mlmmjunsub)
 		
 		if(childpid < 0) {
 			log_error(LOG_ARGS, "Could not fork");
+			myfree(address);
+			myfree(bouncedata);
 			continue;
 		}
 
@@ -754,10 +782,11 @@ int unsub_bouncers(const char *listdir, const char *mlmmjunsub)
 		} else {
 			execlp(mlmmjunsub, mlmmjunsub,
 					"-L", listdir,
-					"-a", address, (char *)NULL);
+					"-b", "-a", address, (char *)NULL);
 			log_error(LOG_ARGS, "Could not execlp %s",
 						mlmmjunsub);
-			return 1;
+			/* This is the child. Exit on failure. */
+			exit(EXIT_FAILURE);
 		}
 	}
 	closedir(bouncedir);
@@ -1078,6 +1107,9 @@ int main(int argc, char **argv)
 		if(chdir(dirlists) < 0) {
 			log_error(LOG_ARGS, "Could not chdir(%s).",
 					dirlists);
+			myfree(mlmmjbounce);
+			myfree(mlmmjsend);
+			myfree(mlmmjunsub);
 			exit(EXIT_FAILURE);
 		}
 
