@@ -1,6 +1,6 @@
 /* Copyright (C) 2003, 2003, 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-process.c,v 1.86 2009/07/27 11:55:27 mmj Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -40,7 +40,7 @@
 #include "getlistdelim.h"
 #include "listcontrol.h"
 #include "strgen.h"
-#include "do_all_the_voodo_here.h"
+#include "do_all_the_voodoo_here.h"
 #include "log_error.h"
 #include "mygetline.h"
 #include "statctrl.h"
@@ -52,9 +52,11 @@
 #include "memory.h"
 #include "log_oper.h"
 #include "chomp.h"
+#include "unistr.h"
 
 enum action {
 	ALLOW,
+	SEND,
 	DENY,
 	MODERATE,
 	DISCARD
@@ -63,21 +65,40 @@ enum action {
 
 static char *action_strs[] = {
 	"allowed",
+	"sent",
 	"denied",
 	"moderated",
 	"discarded"
 };
 
 
-void newmoderated(const char *listdir, const char *mailfilename,
-		  const char *mlmmjsend, const char *efromsender)
+enum modreason {
+	MODNONSUBPOSTS,
+	ACCESS,
+	MODERATED
+};
+
+
+static char *modreason_strs[] = {
+	"modnonsubposts",
+	"access",
+	"moderated"
+};
+
+
+static void newmoderated(const char *listdir, const char *mailfilename,
+		  const char *mlmmjsend, const char *efromsender,
+		  const char *subject, const char *posteraddr,
+		  enum modreason modreason)
 {
 	char *from, *listfqdn, *listname, *moderators = NULL;
 	char *buf, *replyto, *listaddr = getlistaddr(listdir), *listdelim;
+	text *txt;
+	memory_lines_state *mls;
 	char *queuefilename = NULL, *moderatorsfilename, *efromismod = NULL;
-	char *mailbasename = mybasename(mailfilename), *tmp, *to;
-	int moderatorsfd, foundaddr = 0;
-	char *maildata[4] = { "moderateaddr", NULL, "moderators", NULL };
+	char *mailbasename = mybasename(mailfilename), *tmp, *to, *reject;
+	int moderatorsfd, foundaddr = 0, notifymod = 0, status;
+	pid_t childpid, pid;
 #if 0
 	printf("mailfilename = [%s], mailbasename = [%s]\n", mailfilename,
 			                                     mailbasename);
@@ -110,18 +131,17 @@ void newmoderated(const char *listdir, const char *mailfilename,
 		efromismod = NULL;
 	}
 
+	if(efromismod) mls = init_memory_lines(efromismod);
+	else mls = init_memory_lines(moderators);
+
 	close(moderatorsfd);
+	myfree(moderators);
 
 	listdelim = getlistdelim(listdir);
-	replyto = concatstr(6, listname, listdelim, "moderate-", mailbasename,
+	replyto = concatstr(6, listname, listdelim, "release-", mailbasename,
+			    "@", listfqdn);
+	reject = concatstr(6, listname, listdelim, "reject-", mailbasename,
 			    "@", listfqdn);
-
-	maildata[1] = replyto;
-	if(efromismod) {
-		myfree(moderators);
-		maildata[3] = efromismod;
-	} else
-		maildata[3] = moderators;
 
 	from = concatstr(4, listname, listdelim, "owner@", listfqdn);
 	to = concatstr(3, listname, "-moderators@", listfqdn); /* FIXME JFA: Should this be converted? Why, why not? */
@@ -130,25 +150,87 @@ void newmoderated(const char *listdir, const char *mailfilename,
 	myfree(listname);
 	myfree(listfqdn);
 
-	queuefilename = prepstdreply(listdir, "moderation", "$listowner$",
-				     to, replyto, 2, maildata, NULL, mailfilename);
+	txt = open_text(listdir, "moderate", "post",
+			modreason_strs[modreason], NULL, "moderation");
+	MY_ASSERT(txt);
+	register_unformatted(txt, "subject", subject);
+	register_unformatted(txt, "posteraddr", posteraddr);
+	register_unformatted(txt, "moderateaddr", replyto); /* DEPRECATED */
+	register_unformatted(txt, "releaseaddr", replyto);
+	register_unformatted(txt, "rejectaddr", reject);
+	register_unformatted(txt, "moderators", "%moderators%"); /* DEPRECATED */
+	register_formatted(txt, "moderators",
+			rewind_memory_lines, get_memory_line, mls);
+	register_originalmail(txt, mailfilename);
+	queuefilename = prepstdreply(txt, listdir, "$listowner$", to, replyto);
+	MY_ASSERT(queuefilename);
+	close_text(txt);
 
-	if(efromismod)
-		execlp(mlmmjsend, mlmmjsend,
-				"-l", "1",
-				"-L", listdir,
-				"-F", from,
-				"-m", queuefilename,
-				"-T", efromsender, (char *)NULL);
-	else
-		execlp(mlmmjsend, mlmmjsend,
-				"-l", "2",
-				"-L", listdir,
-				"-F", from,
-				"-m", queuefilename, (char *)NULL);
+	/* we might need to exec more than one mlmmj-send */
+	
+	notifymod = !efromismod && statctrl(listdir,"notifymod");
+	
+	if (notifymod) {
+		childpid = fork();
+		if(childpid < 0)
+			log_error(LOG_ARGS, "Could not fork; poster not notified");
+	} else
+		childpid = -1;
+
+	if(childpid != 0) {
+		if(childpid > 0) {
+			do /* Parent waits for the child */
+				pid = waitpid(childpid, &status, 0);
+			while(pid == -1 && errno == EINTR);
+		}
+
+		finish_memory_lines(mls);
+
+		if(efromismod)
+			execlp(mlmmjsend, mlmmjsend,
+					"-l", "1",
+					"-L", listdir,
+					"-F", from,
+					"-m", queuefilename,
+					"-T", efromsender, (char *)NULL);
+		else
+			execlp(mlmmjsend, mlmmjsend,
+					"-l", "2",
+					"-L", listdir,
+					"-F", from,
+					"-m", queuefilename, (char *)NULL);
+		log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
+		exit(EXIT_FAILURE);
+	}
+
+	myfree(queuefilename);
+
+	/* send mail to poster that the list is moderated */
+
+	txt = open_text(listdir, "wait", "post",
+			modreason_strs[modreason], NULL, "moderation-poster");
+	MY_ASSERT(txt);
+	register_unformatted(txt, "subject", subject);
+	register_unformatted(txt, "posteraddr", posteraddr);
+	register_unformatted(txt, "moderators", "%moderators%"); /* DEPRECATED */
+	register_formatted(txt, "moderators",
+			rewind_memory_lines, get_memory_line, mls);
+	register_originalmail(txt, mailfilename);
+	queuefilename = prepstdreply(txt, listdir,
+			"$listowner$", efromsender, NULL);
+	MY_ASSERT(queuefilename);
+	close_text(txt);
+
+	finish_memory_lines(mls);
+
+	execlp(mlmmjsend, mlmmjsend,
+			"-l", "1",
+			"-L", listdir,
+			"-F", from,
+			"-m", queuefilename,
+			"-T", efromsender, (char *)NULL);
 
 	log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
-
 	exit(EXIT_FAILURE);
 }
 
@@ -173,6 +255,9 @@ static enum action do_access(struct strlist *rule_strs, struct strlist *hdrs,
 		if (strncmp(rule_ptr, "allow", 5) == 0) {
 			rule_ptr += 5;
 			act = ALLOW;
+		} else if (strncmp(rule_ptr, "send", 4) == 0) {
+			rule_ptr += 4;
+			act = SEND;
 		} else if (strncmp(rule_ptr, "deny", 4) == 0) {
 			rule_ptr += 4;
 			act = DENY;
@@ -318,7 +403,8 @@ static char *recipient_extra(const char *listdir, const char *addr)
 
 static void print_help(const char *prg)
 {
-        printf("Usage: %s -L /path/to/list -m /path/to/mail [-h] [-P] [-V]\n"
+	printf("Usage: %s -L /path/to/list\n"
+	       "       -m /path/to/mail [-h] [-P] [-V]\n"
 	       " -h: This help\n"
 	       " -L: Full path to list directory\n"
 	       " -m: Full path to mail file\n"
@@ -330,21 +416,22 @@ static void print_help(const char *prg)
 
 int main(int argc, char **argv)
 {
-	int i, j, opt, noprocess = 0, moderated = 0;
-	int hdrfd, footfd, rawmailfd, donemailfd;
-	int subonlypost = 0, addrtocc = 1, intocc = 0, modnonsubposts = 0;
+	int i, j, opt, noprocess = 0, moderated = 0, send = 0;
+	enum modreason modreason;
+	int hdrfd, footfd, rawmailfd, donemailfd, omitfd;
+	int addrtocc = 1, intocc = 0;
 	int maxmailsize = 0;
-	int notoccdenymails = 0, noaccessdenymails = 0, nosubonlydenymails = 0;
-	int nomaxmailsizedenymails = 0;
+	int notmetoo = 0;
 	char *listdir = NULL, *mailfile = NULL, *headerfilename = NULL;
 	char *footerfilename = NULL, *donemailname = NULL;
-	char *randomstr = NULL, *mqueuename;
+	char *randomstr = NULL, *mqueuename, *omitfilename;
 	char *mlmmjsend, *mlmmjsub, *mlmmjunsub, *mlmmjbounce;
 	char *bindir, *subjectprefix, *discardname, *listaddr, *listdelim;
 	char *listfqdn, *listname, *fromaddr;
+	text *txt;
 	char *queuefilename, *recipextra = NULL, *owner = NULL;
 	char *maxmailsizestr;
-	char *maildata[4] = { "posteraddr", NULL, "maxmailsize", NULL };
+	char *subject = NULL, *posteraddr = NULL;
 	char *envstr, *efrom;
 	struct stat st;
 	uid_t uid;
@@ -363,6 +450,7 @@ int main(int argc, char **argv)
 		{ "Cc:", 0, NULL },
 		{ "Return-Path:", 0, NULL },
 		{ "Delivered-To:", 0, NULL },
+		{ "Subject:", 0, NULL },
 		{ NULL, 0, NULL }
 	};
 
@@ -448,13 +536,13 @@ int main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
-    /* hdrfd is checked in do_all_the_voodo_here(), because the
-     * customheaders file might not exist */
+	/* hdrfd is checked in do_all_the_voodoo_here(), because the
+	 * customheaders file might not exist */
 	headerfilename = concatstr(2, listdir, "/control/customheaders");
 	hdrfd = open(headerfilename, O_RDONLY);
 	myfree(headerfilename);
 
-    /* footfd is checked in do_all_the_voodo_here(), see above */
+	/* footfd is checked in do_all_the_voodoo_here(), see above */
 	footerfilename = concatstr(2, listdir, "/control/footer");
 	footfd = open(footerfilename, O_RDONLY);
 	myfree(footerfilename);
@@ -474,10 +562,10 @@ int main(int argc, char **argv)
 
 	subjectprefix = ctrlvalue(listdir, "prefix");
 
-	if(do_all_the_voodo_here(rawmailfd, donemailfd, hdrfd, footfd,
+	if(do_all_the_voodoo_here(rawmailfd, donemailfd, hdrfd, footfd,
 				(const char**)delheaders->strs, readhdrs,
 				&allheaders, subjectprefix) < 0) {
-		log_error(LOG_ARGS, "Error in do_all_the_voodo_here");
+		log_error(LOG_ARGS, "Error in do_all_the_voodoo_here");
 		exit(EXIT_FAILURE);
 	}
 
@@ -493,11 +581,6 @@ int main(int argc, char **argv)
 	if(footfd >= 0)
 		close(footfd);
 
-	/* From: addresses */
-	for(i = 0; i < readhdrs[0].valuecount; i++) {
-		find_email_adr(readhdrs[0].values[i], &fromemails);
-	}
-
 	/* To: addresses */
 	for(i = 0; i < readhdrs[1].valuecount; i++) {
 		find_email_adr(readhdrs[1].values[i], &toemails);
@@ -508,27 +591,11 @@ int main(int argc, char **argv)
 		find_email_adr(readhdrs[2].values[i], &ccemails);
 	}
 
-	/* Return-Path: addresses */
-	for(i = 0; i < readhdrs[3].valuecount; i++) {
-		find_email_adr(readhdrs[3].values[i], &rpemails);
-	}
-
 	/* Delivered-To: addresses */
 	for(i = 0; i < readhdrs[4].valuecount; i++) {
 		find_email_adr(readhdrs[4].values[i], &dtemails);
 	}
 
-	/* envelope from */
-	if((envstr = getenv("SENDER")) != NULL) {
-		/* qmail, postfix, exim */
-		efrom = mystrdup(envstr);
-	} else if(rpemails.emailcount >= 1) {
-		/* the (first) Return-Path: header */
-		efrom = mystrdup(rpemails.emaillist[0]);
-	} else {
-		efrom = mystrdup("");
-	}
-
 	/* address extension (the "foo" part of "user+foo@domain.tld") */
 	if((envstr = getenv("DEFAULT")) != NULL) {
 		/* qmail */
@@ -554,16 +621,69 @@ int main(int argc, char **argv)
 	} else {
 		recipextra = NULL;
 	}
-
 	if(recipextra && (strlen(recipextra) == 0)) {
 		myfree(recipextra);
 		recipextra = NULL;
 	}
 
-	/* Why is this here, and not in listcontrol() ?  -- mortenp 20060409 */
+	/* From: addresses */
+	for(i = 0; i < readhdrs[0].valuecount; i++) {
+		find_email_adr(readhdrs[0].values[i], &fromemails);
+	}
+	/* discard malformed mail with invalid From: unless it's a bounce */
+	if(fromemails.emailcount != 1 &&
+			(recipextra == NULL ||
+			strncmp(recipextra, "bounces", 7) != 0)) {
+		for(i = 0; i < fromemails.emailcount; i++)
+			printf("fromemails.emaillist[%d] = %s\n",
+					i, fromemails.emaillist[i]);
+		discardname = concatstr(3, listdir,
+				"/queue/discarded/", randomstr);
+		log_error(LOG_ARGS, "Discarding %s due to invalid From:",
+				mailfile);
+		for(i = 0; i < fromemails.emailcount; i++)
+			log_error(LOG_ARGS, "fromemails.emaillist[%d] = %s\n",
+					i, fromemails.emaillist[i]);
+		rename(mailfile, discardname);
+		unlink(donemailname);
+		myfree(donemailname);
+		myfree(discardname);
+		myfree(randomstr);
+		/* TODO: free emailstructs */
+		exit(EXIT_SUCCESS);
+	}
+	/* The only time posteraddr will remain unset is when the mail is a
+	 * bounce, so the mail will be processed by listcontrol() and the
+	 * program will terminate before posteraddr is used. */
+	if (fromemails.emailcount > 0)
+			posteraddr = fromemails.emaillist[0];
+
+	/* Return-Path: addresses */
+	for(i = 0; i < readhdrs[3].valuecount; i++) {
+		find_email_adr(readhdrs[3].values[i], &rpemails);
+	}
+
+	/* envelope from */
+	if((envstr = getenv("SENDER")) != NULL) {
+		/* qmail, postfix, exim */
+		efrom = mystrdup(envstr);
+	} else if(rpemails.emailcount >= 1) {
+		/* the (first) Return-Path: header */
+		efrom = mystrdup(rpemails.emaillist[0]);
+	} else {
+		efrom = mystrdup("");
+	}
+
+	/* Subject: */
+	if (readhdrs[5].valuecount)
+			subject = unistr_header_to_utf8(readhdrs[5].values[0]);
+	if (!subject) subject = mystrdup("");
+
 	if(recipextra) {
 		owner = concatstr(2, listdir, "/control/owner");
 		if(owner && strcmp(recipextra, "owner") == 0) {
+			/* Why is this here, and not in listcontrol() ?
+			 * -- mortenp 20060409 */
 			/* strip envelope from before resending */
 			delheaders->count = 0;
 			delheaders->strs = NULL;
@@ -585,16 +705,16 @@ int main(int argc, char **argv)
 						    "output mail file");
 				exit(EXIT_FAILURE);
 			}
-			if(do_all_the_voodo_here(rawmailfd, donemailfd, -1,
+			if(do_all_the_voodoo_here(rawmailfd, donemailfd, -1,
 					-1, (const char**)delheaders->strs,
 					NULL, &allheaders, NULL) < 0) {
-				log_error(LOG_ARGS, "do_all_the_voodo_here");
+				log_error(LOG_ARGS, "do_all_the_voodoo_here");
 				exit(EXIT_FAILURE);
 			}
 			close(rawmailfd);
 			close(donemailfd);
 			unlink(mailfile);
-			log_oper(listdir, OPLOGFNAME, "mlmmj-recieve: sending"
+			log_oper(listdir, OPLOGFNAME, "mlmmj-process: sending"
 					" mail from %s to owner",
 					efrom);
 			execlp(mlmmjsend, mlmmjsend,
@@ -633,8 +753,7 @@ int main(int argc, char **argv)
 
 		if(st.st_size > maxmailsize) {
 
-			nomaxmailsizedenymails = statctrl(listdir, "nomaxmailsizedenymails");
-			if (nomaxmailsizedenymails) {
+			if (statctrl(listdir, "nomaxmailsizedenymails")) {
 				errno = 0;
 				log_error(LOG_ARGS, "Discarding %s due to"
 						" size limit (%d bytes too big)",
@@ -651,12 +770,17 @@ int main(int argc, char **argv)
 			listfqdn = genlistfqdn(listaddr);
 			fromaddr = concatstr(4, listname, listdelim,
 					"bounces-help@", listfqdn);
-			maildata[3] = maxmailsizestr;
-			queuefilename = prepstdreply(listdir,
-					"maxmailsize", "$listowner$",
-					fromemails.emaillist[0],
-					NULL, 2, maildata, NULL, donemailname);
-			MY_ASSERT(queuefilename)
+			txt = open_text(listdir, "deny", "post",
+					"maxmailsize", NULL, "maxmailsize");
+			MY_ASSERT(txt);
+			register_unformatted(txt, "subject", subject);
+			register_unformatted(txt, "posteraddr", posteraddr);
+			register_unformatted(txt, "maxmailsize", maxmailsizestr);
+			register_originalmail(txt, donemailname);
+			queuefilename = prepstdreply(txt, listdir,
+					"$listowner$", posteraddr, NULL);
+			MY_ASSERT(queuefilename);
+			close_text(txt);
 			myfree(listdelim);
 			myfree(listname);
 			myfree(listfqdn);
@@ -667,7 +791,7 @@ int main(int argc, char **argv)
 			execlp(mlmmjsend, mlmmjsend,
 					"-l", "1",
 					"-L", listdir,
-					"-T", fromemails.emaillist[0],
+					"-T", posteraddr,
 					"-F", fromaddr,
 					"-m", queuefilename, (char *)NULL);
 
@@ -676,27 +800,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	/* discard malformed mail with invalid From: */
-	if(fromemails.emailcount != 1) {
-		for(i = 0; i < fromemails.emailcount; i++)
-			printf("fromemails.emaillist[%d] = %s\n",
-					i, fromemails.emaillist[i]);
-		discardname = concatstr(3, listdir,
-				"/queue/discarded/", randomstr);
-		log_error(LOG_ARGS, "Discarding %s due to invalid From:",
-				mailfile);
-		for(i = 0; i < fromemails.emailcount; i++)
-			log_error(LOG_ARGS, "fromemails.emaillist[%d] = %s\n",
-					i, fromemails.emaillist[i]);
-		rename(mailfile, discardname);
-		unlink(donemailname);
-		myfree(donemailname);
-		myfree(discardname);
-		myfree(randomstr);
-		/* TODO: free emailstructs */
-		exit(EXIT_SUCCESS);
-	}
-
 	myfree(delheaders);
 
 	if(strcmp(efrom, "") == 0) { /* don't send mails with <> in From
@@ -721,6 +824,7 @@ int main(int argc, char **argv)
 	addrtocc = !(statctrl(listdir, "tocc"));
 	if(addrtocc) {
 		for(i = 0; i < toemails.emailcount; i++) {
+			errno = 0;
 			log_error(LOG_ARGS, "Found To: %s",
 				toemails.emaillist[i]);
 			for(j = 0; j < alternates->count; j++) {
@@ -731,6 +835,7 @@ int main(int argc, char **argv)
 			}
 		}
 		for(i = 0; i < ccemails.emailcount; i++) {
+			errno = 0;
 			log_error(LOG_ARGS, "Found Cc: %s",
 				ccemails.emaillist[i]);
 			for(j = 0; j < alternates->count; j++) {
@@ -745,14 +850,12 @@ int main(int argc, char **argv)
 	for(i = 0; i < alternates->count; i++)
 		myfree(alternates->strs[i]);
 
-	notoccdenymails = statctrl(listdir, "notoccdenymails");
-
 	if(addrtocc && !intocc) {
 		/* Don't send a mail about denial to the list, but silently
 		 * discard and exit. Also don't in case of it being turned off
 		 */
-		if ((strcasecmp(listaddr, fromemails.emaillist[0]) == 0) ||
-				notoccdenymails) {
+		if ((strcasecmp(listaddr, posteraddr) == 0) ||
+				statctrl(listdir, "notoccdenymails")) {
 			log_error(LOG_ARGS, "Discarding %s because list"
 					" address was not in To: or Cc:,"
 					" and From: was the list or"
@@ -768,10 +871,16 @@ int main(int argc, char **argv)
 		listfqdn = genlistfqdn(listaddr);
 		fromaddr = concatstr(4, listname, listdelim, "bounces-help@",
 				     listfqdn);
-		queuefilename = prepstdreply(listdir, "notintocc",
-					"$listowner$", fromemails.emaillist[0],
-					     NULL, 0, NULL, NULL, donemailname);
+		txt = open_text(listdir, "deny", "post",
+				"notintocc", NULL, "notintocc");
+		MY_ASSERT(txt);
+		register_unformatted(txt, "subject", subject);
+		register_unformatted(txt, "posteraddr", posteraddr);
+		register_originalmail(txt, donemailname);
+		queuefilename = prepstdreply(txt, listdir,
+				"$listowner$", posteraddr, NULL);
 		MY_ASSERT(queuefilename)
+		close_text(txt);
 		myfree(listdelim);
 		myfree(listname);
 		myfree(listfqdn);
@@ -780,7 +889,7 @@ int main(int argc, char **argv)
 		execlp(mlmmjsend, mlmmjsend,
 				"-l", "1",
 				"-L", listdir,
-				"-T", fromemails.emaillist[0],
+				"-T", posteraddr,
 				"-F", fromaddr,
 				"-m", queuefilename, (char *)NULL);
 
@@ -788,87 +897,22 @@ int main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
-	subonlypost = statctrl(listdir, "subonlypost");
-	if(subonlypost) {
-		/* Don't send a mail about denial to the list, but silently
-		 * discard and exit. */
-		if (strcasecmp(listaddr, fromemails.emaillist[0]) == 0) {
-			log_error(LOG_ARGS, "Discarding %s because"
-					" subonlypost was set and From: was"
-					" the list address",
-					mailfile);
-			myfree(listaddr);
-			unlink(donemailname);
-			myfree(donemailname);
-			exit(EXIT_SUCCESS);
-		}
-		if(is_subbed(listdir, fromemails.emaillist[0]) != 0) {
-			modnonsubposts = statctrl(listdir,
-					"modnonsubposts");
-			if(modnonsubposts) {
-				moderated = 1;
-				goto startaccess;
-			}
-
-			nosubonlydenymails = statctrl(listdir,
-					"nosubonlydenymails");
-
-			if(nosubonlydenymails) {
-				log_error(LOG_ARGS, "Discarding %s because"
-						" subonlypost and"
-						" nosubonlydenymails was set",
-						mailfile);
-				myfree(listaddr);
-				unlink(donemailname);
-				myfree(donemailname);
-				exit(EXIT_SUCCESS);
-			}
-			listdelim = getlistdelim(listdir);
-			listname = genlistname(listaddr);
-			listfqdn = genlistfqdn(listaddr);
-			maildata[1] = fromemails.emaillist[0];
-			fromaddr = concatstr(4, listname, listdelim,
-					"bounces-help@", listfqdn);
-			queuefilename = prepstdreply(listdir, "subonlypost",
-					"$listowner$", fromemails.emaillist[0],
-						     NULL, 1, maildata, NULL, donemailname);
-			MY_ASSERT(queuefilename)
-			myfree(listaddr);
-			myfree(listdelim);
-			myfree(listname);
-			myfree(listfqdn);
-			unlink(donemailname);
-			myfree(donemailname);
-			execlp(mlmmjsend, mlmmjsend,
-					"-l", "1",
-					"-T", fromemails.emaillist[0],
-					"-F", fromaddr,
-					"-m", queuefilename, (char *)NULL);
-
-			log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
-			exit(EXIT_FAILURE);
-		}
-	}
-
-startaccess:
-	noaccessdenymails = statctrl(listdir, "noaccessdenymails");
-
 	access_rules = ctrlvalues(listdir, "access");
 	if (access_rules) {
 		enum action accret;
 		/* Don't send a mail about denial to the list, but silently
 		 * discard and exit. Also do this in case it's turned off */
 		accret = do_access(access_rules, &allheaders,
-					fromemails.emaillist[0], listdir);
+					posteraddr, listdir);
 		if (accret == DENY) {
-			if ((strcasecmp(listaddr, fromemails.emaillist[0]) ==
-						0) || noaccessdenymails) {
+			if ((strcasecmp(listaddr, posteraddr) == 0) ||
+				    statctrl(listdir, "noaccessdenymails")) {
 				log_error(LOG_ARGS, "Discarding %s because"
-						" it was denied by an access"
-						" rule, and From: was the list"
-						" address or noaccessdenymails"
-						" was set",
-						mailfile);
+					" it was denied by an access"
+					" rule, and From: was the list"
+					" address or noaccessdenymails"
+					" was set",
+					mailfile);
 				myfree(listaddr);
 				unlink(donemailname);
 				myfree(donemailname);
@@ -879,11 +923,16 @@ startaccess:
 			listfqdn = genlistfqdn(listaddr);
 			fromaddr = concatstr(4, listname, listdelim,
 					"bounces-help@", listfqdn);
-			queuefilename = prepstdreply(listdir, "access",
-							"$listowner$",
-							fromemails.emaillist[0],
-						     NULL, 0, NULL, NULL, donemailname);
+			txt = open_text(listdir, "deny", "post",
+					"access", NULL, "access");
+			MY_ASSERT(txt);
+			register_unformatted(txt, "subject", subject);
+			register_unformatted(txt, "posteraddr", posteraddr);
+			register_originalmail(txt, donemailname);
+			queuefilename = prepstdreply(txt, listdir,
+					"$listowner$", posteraddr, NULL);
 			MY_ASSERT(queuefilename)
+			close_text(txt);
 			myfree(listaddr);
 			myfree(listdelim);
 			myfree(listname);
@@ -894,7 +943,7 @@ startaccess:
 			execlp(mlmmjsend, mlmmjsend,
 					"-l", "1",
 					"-L", listdir,
-					"-T", fromemails.emaillist[0],
+					"-T", posteraddr,
 					"-F", fromaddr,
 					"-m", queuefilename, (char *)NULL);
 
@@ -903,6 +952,7 @@ startaccess:
 			exit(EXIT_FAILURE);
 		} else if (accret == MODERATE) {
 			moderated = 1;
+			modreason = ACCESS;
 		} else if (accret == DISCARD) {
 	                discardname = concatstr(3, listdir,
                                 "/queue/discarded/", randomstr);
@@ -917,11 +967,84 @@ startaccess:
 			myfree(donemailname);
 			myfree(discardname);
                 	exit(EXIT_SUCCESS);
+		} else if (accret == SEND) {
+			send = 1;
+		} else if (accret == ALLOW) {
+			/* continue processing as normal */
 		}
 	}
 
-	if(!moderated)
-		moderated = statctrl(listdir, "moderated");
+	if(!send && (statctrl(listdir, "subonlypost") ||
+			statctrl(listdir, "modnonsubposts"))) {
+		/* Don't send a mail about denial to the list, but silently
+		 * discard and exit. */
+		if (strcasecmp(listaddr, posteraddr) == 0) {
+			log_error(LOG_ARGS, "Discarding %s because"
+					" subonlypost was set and From: was"
+					" the list address",
+					mailfile);
+			myfree(listaddr);
+			unlink(donemailname);
+			myfree(donemailname);
+			exit(EXIT_SUCCESS);
+		}
+		if(is_subbed(listdir, posteraddr, 0) == SUB_NONE) {
+			if(statctrl(listdir, "modnonsubposts")) {
+				moderated = 1;
+				modreason = MODNONSUBPOSTS;
+			} else {
+				if(statctrl(listdir, "nosubonlydenymails")) {
+				    log_error(LOG_ARGS, "Discarding %s because"
+					    " subonlypost and"
+					    " nosubonlydenymails was set",
+					    mailfile);
+				    myfree(listaddr);
+				    unlink(donemailname);
+				    myfree(donemailname);
+				    exit(EXIT_SUCCESS);
+				}
+				listdelim = getlistdelim(listdir);
+				listname = genlistname(listaddr);
+				listfqdn = genlistfqdn(listaddr);
+				fromaddr = concatstr(4, listname, listdelim,
+					"bounces-help@", listfqdn);
+				txt = open_text(listdir, "deny", "post",
+					"subonlypost", NULL, "subonlypost");
+				MY_ASSERT(txt);
+				register_unformatted(txt, "subject", subject);
+				register_unformatted(txt, "posteraddr", posteraddr);
+				register_originalmail(txt, donemailname);
+				queuefilename = prepstdreply(txt, listdir,
+					"$listowner$", posteraddr, NULL);
+				MY_ASSERT(queuefilename)
+				close_text(txt);
+				myfree(listaddr);
+				myfree(listdelim);
+				myfree(listname);
+				myfree(listfqdn);
+				unlink(donemailname);
+				myfree(donemailname);
+				execlp(mlmmjsend, mlmmjsend,
+					"-l", "1",
+					"-T", posteraddr,
+					"-F", fromaddr,
+					"-m", queuefilename, (char *)NULL);
+
+				log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
+				exit(EXIT_FAILURE);
+			}
+		}
+	}
+
+	if(!send && !moderated) {
+		if(statctrl(listdir, "moderated")) {
+			moderated = 1;
+			modreason = MODERATED;
+		}
+	}
+
+	notmetoo = statctrl(listdir, "notmetoo");
+
 	if(moderated) {
 		mqueuename = concatstr(3, listdir, "/moderation/",
 				       randomstr);
@@ -934,7 +1057,29 @@ startaccess:
 			exit(EXIT_FAILURE);
 		}
 		myfree(donemailname);
-		newmoderated(listdir, mqueuename, mlmmjsend, efrom);
+		if (notmetoo) {
+			omitfilename = concatstr(2, mqueuename, ".omit");
+			omitfd = open(omitfilename, O_RDWR|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR);
+			if (omitfd < 0) {
+				log_error(LOG_ARGS, "could not open %s",
+					    	    omitfilename);
+				myfree(mqueuename);
+				myfree(omitfilename);
+				exit(EXIT_FAILURE);
+			}
+			myfree(omitfilename);
+			if(writen(omitfd, posteraddr, strlen(posteraddr)) < 0) {
+				log_error(LOG_ARGS,
+						"could not write omit file");
+				myfree(mqueuename);
+				exit(EXIT_FAILURE);
+			}
+			fsync(omitfd);
+			close(omitfd);
+		}
+		newmoderated(listdir, mqueuename,
+				mlmmjsend, efrom, subject, posteraddr,
+				modreason);
 		return EXIT_SUCCESS;
 	}
 
@@ -946,7 +1091,13 @@ startaccess:
 		exit(EXIT_SUCCESS);
 	}
 
-	execlp(mlmmjsend, mlmmjsend,
+	if (notmetoo)
+		execlp(mlmmjsend, mlmmjsend,
+				"-L", listdir,
+				"-o", posteraddr,
+				"-m", donemailname, (char *)NULL);
+	else
+		execlp(mlmmjsend, mlmmjsend,
 				"-L", listdir,
 				"-m", donemailname, (char *)NULL);
 	log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
