@@ -1,6 +1,6 @@
 /* Copyright (C) 2004, 2003, 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-send.c,v 1.90 2008/10/31 15:43:37 mortenp Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -248,7 +248,10 @@ int send_mail(int sockfd, const char *from, const char *to,
 	      size_t bodylen)
 {
 	int retval = 0;
-	char *reply, *tohdr;
+	char *reply, *reply2, *tohdr;
+
+	if(sockfd == -1)
+		return EBADF;
 
 	if(strchr(to, '@') == NULL) {
 		errno = 0;
@@ -268,7 +271,8 @@ int send_mail(int sockfd, const char *from, const char *to,
 				reply);
 		myfree(reply);
 		write_rset(sockfd);
-		checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		reply2 = checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		if (reply2 != NULL) myfree(reply2);
 		return MLMMJ_FROM;
 	}
 	retval = write_rcpt_to(sockfd, to);
@@ -280,7 +284,8 @@ int send_mail(int sockfd, const char *from, const char *to,
 	reply = checkwait_smtpreply(sockfd, MLMMJ_RCPTTO);
 	if(reply) {
 		write_rset(sockfd);
-		checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		reply2 = checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		if (reply2 != NULL) myfree(reply2);
 		if(mlmmjbounce && ((reply[0] == '4') || (reply[0] == '5'))
 				&& (reply[1] == '5')) {
 			myfree(reply);
@@ -304,7 +309,8 @@ int send_mail(int sockfd, const char *from, const char *to,
 		log_error(LOG_ARGS, "Error with DATA. Reply = [%s]", reply);
 		myfree(reply);
 		write_rset(sockfd);
-		checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		reply2 = checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		if (reply2 != NULL) myfree(reply2);
 		return MLMMJ_DATA;
 	}
 
@@ -362,7 +368,8 @@ int send_mail(int sockfd, const char *from, const char *to,
 				"avail. Reply = [%s]", reply);
 		myfree(reply);
 		write_rset(sockfd);
-		checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		reply2 = checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		if (reply2 != NULL) myfree(reply2);
 		return MLMMJ_DOT;
 	}
 
@@ -374,9 +381,12 @@ int initsmtp(int *sockfd, const char *relayhost, unsigned short port)
 	int retval = 0;
 	char *reply = NULL;
 	char *myhostname = hostnamestr();
-	
+
 	init_sockfd(sockfd, relayhost, port);
-	
+
+	if(*sockfd == -1)
+		return EBADF;
+
 	if((reply = checkwait_smtpreply(*sockfd, MLMMJ_CONNECT)) != NULL) {
 		log_error(LOG_ARGS, "No proper greeting to our connect"
 			  "Reply: [%s]", reply);
@@ -416,7 +426,6 @@ int endsmtp(int *sockfd)
 	}
 
 	close(*sockfd);
-
 	*sockfd = -1;
 
 	return retval;
@@ -428,7 +437,10 @@ int send_mail_verp(int sockfd, struct strlist *addrs, char *mailmap,
 		   size_t bodylen, const char *verpextra)
 {
 	int retval, i;
-	char *reply;
+	char *reply, *reply2;
+
+	if(sockfd == -1)
+		return EBADF;
 
 	retval = write_mail_from(sockfd, from, verpextra);
 	if(retval) {
@@ -441,12 +453,13 @@ int send_mail_verp(int sockfd, struct strlist *addrs, char *mailmap,
 				reply);
 		myfree(reply);
 		write_rset(sockfd);
-		checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		reply2 = checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		if (reply2 != NULL) myfree(reply2);
 		return MLMMJ_FROM;
 	}
 	for(i = 0; i < addrs->count; i++) {
 		if(gotsigterm) {
-			log_error(LOG_ARGS, "TERM signal recieved, "
+			log_error(LOG_ARGS, "TERM signal received, "
 					"shutting down.");
 			return -1;
 		}
@@ -482,7 +495,8 @@ int send_mail_verp(int sockfd, struct strlist *addrs, char *mailmap,
 		log_error(LOG_ARGS, "Error with DATA. Reply = [%s]", reply);
 		myfree(reply);
 		write_rset(sockfd);
-		checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		reply2 = checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		if (reply2 != NULL) myfree(reply2);
 		return MLMMJ_DATA;
 	}
 
@@ -519,7 +533,8 @@ int send_mail_verp(int sockfd, struct strlist *addrs, char *mailmap,
 				"avail. Reply = [%s]", reply);
 		myfree(reply);
 		write_rset(sockfd);
-		checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		reply2 = checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		if (reply2 != NULL) myfree(reply2);
 		return MLMMJ_DOT;
 	}
 
@@ -640,7 +655,7 @@ int send_mail_many_list(int sockfd, const char *from, const char *replyto,
 		}
 		if(gotsigterm && listaddr && archivefilename) {
 			/* we got SIGTERM, so save the addresses and bail */
-			log_error(LOG_ARGS, "TERM signal recieved, "
+			log_error(LOG_ARGS, "TERM signal received, "
 						"shutting down.");
 			index = get_index_from_filename(archivefilename);
 			status = requeuemail(listdir, index, addrs, i);
@@ -672,9 +687,10 @@ int send_mail_many_list(int sockfd, const char *from, const char *replyto,
 
 static void print_help(const char *prg)
 {
-        printf("Usage: %s [-L /path/to/list || -l listctrl] \n"
-	       "       -m /path/to/mail [-a] [-D] [-F] [-h] [-r] [-R] "
-	       "[-s] [-T] [-V]\n"
+	printf("Usage: %s [-L /path/to/list -m /path/to/mail | -l listctrl]\n"
+	       "       [-a] [-D] [-F sender@example.org] [-h] [-o address@example.org]\n"
+	       "       [-r 127.0.0.1] [-R reply@example.org] [-s /path/to/subscribers]\n"
+	       "       [-T recipient@example.org] [-V]\n"
 	       " -a: Don't archive the mail\n"
 	       " -D: Don't delete the mail after it's sent\n"
 	       " -F: What to use as MAIL FROM:\n"
@@ -686,9 +702,10 @@ static void print_help(const char *prg)
 	       "    '4' means 'send to file with recipients'\n"
 	       "    '5' means 'bounceprobe'\n"
 	       "    '6' means 'single listmail to single recipient'\n"
-	       "    '7' means 'digest'\n"
-	       " -L: Full path to list directory\n"
+	       "    '7' means 'digest'\n");
+	printf(" -L: Full path to list directory\n"
 	       " -m: Full path to mail file\n"
+	       " -o: Address to omit from distribution (normal mail only)\n"
 	       " -r: Relayhost IP address (defaults to 127.0.0.1)\n"
 	       " -R: What to use as Reply-To: header\n"
 	       " -s: Subscribers file name\n"
@@ -704,7 +721,7 @@ int main(int argc, char **argv)
 	int deletewhensent = 1, sendres = 0, archive = 1, digest = 0;
 	int ctrlarchive, res;
 	char *listaddr = NULL, *listdelim = NULL;
-	char *mailfilename = NULL, *subfilename = NULL;
+	char *mailfilename = NULL, *subfilename = NULL, *omit = NULL;
 	char *replyto = NULL, *bounceaddr = NULL, *to_addr = NULL;
 	char *relayhost = NULL, *archivefilename = NULL, *tmpstr;
 	char *listctrl = NULL, *subddirname = NULL, *listdir = NULL;
@@ -737,7 +754,7 @@ int main(int argc, char **argv)
 	if(sigaction(SIGTERM, &sigact, NULL) < 0)
 		log_error(LOG_ARGS, "Could not install SIGTERM handler!");
 
-	while ((opt = getopt(argc, argv, "aVDhm:l:L:R:F:T:r:s:")) != -1){
+	while ((opt = getopt(argc, argv, "aVDhm:l:L:R:F:T:r:s:o:")) != -1){
 		switch(opt) {
 		case 'a':
 			archive = 0;
@@ -760,6 +777,9 @@ int main(int argc, char **argv)
 		case 'm':
 			mailfilename = optarg;
 			break;
+		case 'o':
+			omit = optarg;
+			break;
 		case 'r':
 			relayhost = optarg;
 			break;
@@ -996,11 +1016,11 @@ int main(int argc, char **argv)
 	case '1': /* A single mail is to be sent */
 	case '6':
 		initsmtp(&sockfd, relay, smtpport);
-		sendres = send_mail(sockfd, bounceaddr, to_addr, replyto,
+		if(send_mail(sockfd, bounceaddr, to_addr, replyto,
 				mailmap, st.st_size, listdir, NULL,
-				hdrs, hdrslen, body, bodylen);
-		endsmtp(&sockfd);
-		if(sendres) {
+				hdrs, hdrslen, body, bodylen)) {
+			close(sockfd);
+			sockfd = -1;
 			/* error, so keep it in the queue */
 			deletewhensent = 0;
 			/* dump data we want when resending first check
@@ -1048,6 +1068,8 @@ int main(int argc, char **argv)
 				}
 				close(tmpfd);
 			}
+		} else {
+			endsmtp(&sockfd);
 		}
 		break;
 	case '2': /* Moderators */
@@ -1055,20 +1077,24 @@ int main(int argc, char **argv)
 		if(send_mail_many_fd(sockfd, bounceaddr, NULL, mailmap,
 				     st.st_size, subfd, NULL, NULL, NULL,
 				     listdir, NULL, hdrs, hdrslen,
-				     body, bodylen))
+				     body, bodylen)) {
 			close(sockfd);
-		else
+			sockfd = -1;
+		} else {
 			endsmtp(&sockfd);
+		}
 		break;
 	case '3': /* resending earlier failed mails */
 		initsmtp(&sockfd, relay, smtpport);
 		if(send_mail_many_fd(sockfd, NULL, NULL, mailmap, st.st_size,
 				subfd, listaddr, listdelim, mailfilename,
 				listdir, mlmmjbounce, hdrs, hdrslen,
-				body, bodylen))
+				body, bodylen)) {
 			close(sockfd);
-		else
+			sockfd = -1;
+		} else {
 			endsmtp(&sockfd);
+		}
 		unlink(subfilename);
 		break;
 	case '4': /* send mails to owner */
@@ -1076,18 +1102,20 @@ int main(int argc, char **argv)
 		if(send_mail_many_fd(sockfd, bounceaddr, NULL, mailmap,
 				st.st_size, subfd, listaddr, listdelim,
 				mailfilename, listdir, mlmmjbounce,
-				hdrs, hdrslen, body, bodylen))
+				hdrs, hdrslen, body, bodylen)) {
 			close(sockfd);
-		else
+			sockfd = -1;
+		} else {
 			endsmtp(&sockfd);
+		}
 		break;
 	case '5': /* bounceprobe - handle relayhost local users bouncing*/
 		initsmtp(&sockfd, relay, smtpport);
-		sendres = send_mail(sockfd, bounceaddr, to_addr, replyto,
+		if(send_mail(sockfd, bounceaddr, to_addr, replyto,
 				mailmap, st.st_size, listdir, NULL,
-				hdrs, hdrslen, body, bodylen);
-		endsmtp(&sockfd);
-		if(sendres) {
+				hdrs, hdrslen, body, bodylen)) {
+			close(sockfd);
+			sockfd = -1;
 			/* error, so remove the probefile */
 			tmpstr = mystrdup(to_addr);
 			a = strchr(tmpstr, '@');
@@ -1098,6 +1126,8 @@ int main(int argc, char **argv)
 			unlink(probefile);
 			myfree(probefile);
 			myfree(tmpstr);
+		} else {
+			endsmtp(&sockfd);
 		}
 		break;
 	case '7':
@@ -1146,22 +1176,33 @@ int main(int argc, char **argv)
 		
 		if(verp) {
 			initsmtp(&sockfd, relay, smtpport);
-			if(write_mail_from(sockfd, verpfrom, verp)) {
+			if(sockfd > -1) {
+			    if(write_mail_from(sockfd, verpfrom, verp)) {
 				log_error(LOG_ARGS,
-						"Could not write MAIL FROM\n");
+					    "Could not write VERP MAIL FROM. "
+					    "Not sending with VERP.");
 				verp = NULL;
-			} else {
+			    } else {
 				reply = checkwait_smtpreply(sockfd, MLMMJ_FROM);
 				if(reply) {
 					log_error(LOG_ARGS,
 						"Mailserver did not "
-						"accept verp mail from. "
+						"accept VERP MAIL FROM. "
 						"Not sending with VERP.");
 					myfree(reply);
 					verp = NULL;
 				}
+			    }
+			    /* We can't be in SMTP DATA state or anything like
+			     * that, so should be able to safely QUIT. */
+			    endsmtp(&sockfd);
+			} else {
+			    log_error(LOG_ARGS,
+				    "Could not connect to "
+				    "write VERP MAIL FROM. "
+				    "Not sending with VERP.");
+			    verp = NULL;
 			}
-			endsmtp(&sockfd);
 		}
 
 		while((dp = readdir(subddir)) != NULL) {
@@ -1177,8 +1218,26 @@ int main(int argc, char **argv)
 				continue;
 			}
 			do {
+				i = stl.count;
 				res = getaddrsfromfd(&stl, subfd,
 						maxverprecips);
+				if(omit != NULL) {
+					while(i < stl.count) {
+						if(strcmp(stl.strs[i], omit)
+							== 0) {
+						    myfree(stl.strs[i]);
+						    stl.count--;
+						    while (i < stl.count) {
+							stl.strs[i] =
+								stl.strs[i+1];
+							i++;
+						    }
+						    stl.strs[stl.count] = NULL;
+						    break;
+						}
+						i++;
+					}
+				}
 				if(stl.count == maxverprecips) {
 					initsmtp(&sockfd, relay, smtpport);
 					if(verp) {
@@ -1208,7 +1267,12 @@ int main(int argc, char **argv)
 								hdrs, hdrslen,
 								body, bodylen);
 					}
-					endsmtp(&sockfd);
+				    	if (sendres) {
+					    	close(sockfd);
+					    	sockfd = -1;
+				    	} else {
+					    	endsmtp(&sockfd);
+				    	}
 					for(i = 0; i < stl.count; i++)
 						myfree(stl.strs[i]);
 					stl.count = 0;
@@ -1236,21 +1300,16 @@ int main(int argc, char **argv)
 						mlmmjbounce, hdrs, hdrslen,
 						body, bodylen);
 			}
-			endsmtp(&sockfd);
+			if (sendres) {
+				close(sockfd);
+				sockfd = -1;
+			} else {
+				endsmtp(&sockfd);
+			}
 			for(i = 0; i < stl.count; i++)
 				myfree(stl.strs[i]);
 			stl.count = 0;
 		}
-
-		if (sendres) {
-			/* If send_mail_many() failed we close the
-			 * connection to the mail server in a brutal
-			 * manner, because we could be in any state
-			 * (DATA for instance). */
-			close(sockfd);
-		} else {
-			endsmtp(&sockfd);
-		}
 		myfree(stl.strs);
 		myfree(verpfrom);
 		closedir(subddir);
@@ -1258,15 +1317,10 @@ int main(int argc, char **argv)
 		break;
 	}
 	
-	for(i = 0; i < stl.count; i++)
-		myfree(stl.strs[i]);
-	stl.count = 0;
-
 	myfree(listdelim);
 	myfree(hdrs);
 	myfree(body);
 	myfree(mlmmjbounce);
-	close(sockfd);
 	munmap(mailmap, st.st_size);
 	close(mailfd);
 	myfree(verp);
@@ -1282,10 +1336,23 @@ int main(int argc, char **argv)
 		} else {
 			len = strlen(listdir) + 9 + 20 + 9;
 		  	requeuefilename = mymalloc(len);
-		  	snprintf(requeuefilename, len, "%s/requeue/%d/mailfile", listdir,
-				mindex);
-			if (rename(mailfilename, requeuefilename) < 0)
+			snprintf(requeuefilename, len, "%s/requeue/%d",
+				listdir, mindex);
+			if(stat(requeuefilename, &st) < 0) {
+				/* Nothing was requeued and we don't keep
+				 * mail for a noarchive list. */
 				unlink(mailfilename);
+			} else {
+				snprintf(requeuefilename, len,
+					"%s/requeue/%d/mailfile",
+					listdir, mindex);
+				if (rename(mailfilename, requeuefilename) < 0) {
+					log_error(LOG_ARGS,
+							"Could not rename(%s,%s);",
+							mailfilename,
+							requeuefilename);
+				}
+			}
 			myfree(requeuefilename);
 		}
 		myfree(archivefilename);
